package development

import (
	"path"
	"strings"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/cli"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/icebergs/core/code"
	"shylinux.com/x/icebergs/misc/git"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/api"
	"shylinux.com/x/operation/src/development/model"
)

type serve struct {
	Tables
	repos   repos
	order   string `data:"1"`
	imports string `name:"imports" role:"leader,worker"`
	autogen string `name:"autogen name* help* class* table*" icon:"bi bi-plus-square-dotted" role:"leader,worker"`
}

func (s serve) Imports(m *ice.Message, arg ...string) {
	s.spaceCmdy(m, web.CODE_GIT_REPOS, code.INIT, kit.MergeURL2(m.Option(ice.MSG_USERHOST), web.X(m.Option(model.REPOS_NAME))))
	s.spaceImport(m, "gonganxitong", "shylinux.com/x/community")
	s.spaceImport(m, "db/mysql", "shylinux.com/x/mysql-story")
	s.spaceCompile(m)
}
func (s serve) Autogen(m *ice.Message, arg ...string) {
	defer s.SpaceOpsCmdy(m.SpawnSilent(), web.CODE_GIT_REPOS, mdb.CREATE, path.Join(nfs.USR_LOCAL_WORK, m.Option(model.REPOS_NAME)))
	s.SpaceOpsCmdy(m.SpawnSilent(), web.CODE_COMPILE, code.DEVPACK)
	s.spaceCmdy(m, api.GONGANXITONG_SERVICE, code.AUTOGEN, arg)
	s.spaceRepos(m, git.ADD, nfs.SRC)
	s.spaceRepos(m, git.COMMIT, "-am", "add "+m.Option(model.NAME))
	s.spaceRepos(m, git.PUSH)
	s.spaceCompile(m)
}
func (s serve) List(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		return
	}
	switch ReposType(kit.Int(m.Option(model.REPOS_TYPE))) {
	case ReposSystem:
		if s.spaceCmd(m, ctx.COMMAND, api.GONGANXITONG_SERVICE).Length() == 0 {
			if m.IsTech() {
				s.Button(m, "请注入依赖库", s.Imports)
			} else {
				m.Echo("系统未安化")
			}
			return
		}
		s.spaceCmdy(m, nfs.DIR, nfs.SRC, nfs.PATH, nfs.SIZE, mdb.TIME, kit.Dict(
			nfs.DIR_DEEP, ice.TRUE, nfs.DIR_TYPE, nfs.TYPE_CAT, nfs.DIR_REG, nfs.PORTAL_JSON,
		)).Table(func(value ice.Maps) {
			m.Push(model.NAME, strings.Split(value[nfs.PATH], "/")[1])
		}).SortStrR(mdb.TIME)
		if m.IsTech() {
			m.PushAction(s.Open, s.Conf, s.Code, s.Data).Action(s.Autogen).Display("")
			s.Button(m, "请创建服务模块", s.Autogen)
		} else {
			m.Action()
		}
	}
}
func (s serve) Open(m *ice.Message, arg ...string) {
	m.ProcessOpen(s.spaceLink(m, model.REPOS_UID, m.Option(model.REPOS_UID)))
}
func (s serve) Conf(m *ice.Message, arg ...string) {
	m.ProcessOpen(s.spaceLink(m, mdb.VIEW, mdb.TABLE))
}
func (s serve) Code(m *ice.Message, arg ...string) {
	m.ProcessOpen(kit.MergeURL(web.S(m.Option(model.REPOS_NAME))+web.C(code.VIMER), nfs.PATH, nfs.SRC, nfs.FILE, m.Option(model.NAME)+"/"+nfs.PORTAL_GO))
}
func (s serve) Data(m *ice.Message, arg ...string) {
	m.ProcessOpen(kit.MergeURL(web.S(m.Option(model.REPOS_NAME))+web.C("web.code.mysql.query"), aaa.SESS, "mysql", mdb.DATABASE, m.Option(model.NAME)))
}

func init() { ice.TeamCtxCmd(serve{}) }

func (s serve) spaceLink(m *ice.Message, arg ...ice.Any) string {
	return kit.MergeURL(web.S(m.Option(model.REPOS_NAME))+web.C("web.team."+m.Option(model.NAME)+".portal"), arg...)
}
func (s serve) spaceRepos(m *ice.Message, arg ...string) serve {
	s.spaceCmdy(m, cli.SYSTEM, cli.GIT, arg)
	return s
}
func (s serve) spaceImport(m *ice.Message, arg ...string) {
	s.spaceCmdy(m, web.CODE_AUTOGEN, code.IMPORT, arg)
}
func (s serve) spaceCompile(m *ice.Message) {
	if s.spaceCmdy(m, web.CODE_VIMER, code.COMPILE); m.IsErr() {
		return
	}
	m.WaitEvent(web.OPS_SERVER_OPEN, func(msg *ice.Message, arg ...string) bool {
		return msg.Option(mdb.NAME) == m.Option(model.REPOS_NAME)
	})
}
func (s serve) spaceCmdy(m *ice.Message, arg ...ice.Any) *ice.Message {
	if m.IsErr() {
		return m
	}
	m.SetAppend().SetResult()
	return s.SpaceCmdy(m, m.Option(model.REPOS_NAME), arg...)
}
func (s serve) spaceCmd(m *ice.Message, arg ...ice.Any) *ice.Message {
	if m.IsErr() {
		return m
	}
	m.SetAppend().SetResult()
	return s.SpaceCmd(m, m.Option(model.REPOS_NAME), arg...)
}
